/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.example.order.manager.service;

import java.util.Date;

import org.bluestemsoftware.open.eoa.example.order.manager.service.error.SystemError;
import org.bluestemsoftware.open.eoa.example.order.manager.service.fault.OrderNotFoundFault;
import org.bluestemsoftware.open.eoa.example.order.manager.service.fault.OutOfStockFault;
import org.bluestemsoftware.open.eoa.example.order.manager.service.vo.CustomerOrder;
import org.bluestemsoftware.open.eoa.example.order.manager.service.vo.CustomerOrder.Status;
import org.bluestemsoftware.specification.eoa.application.spring.MyOperation;
import org.bluestemsoftware.specification.eoa.component.intrface.rt.ActionContext;
import org.w3c.dom.Element;

/**
 * Encapsulates logic related to order fulfillment within context of order manager application.
 * Note that this service exposes a web-service interface.
 */
public interface OrderProcessor {

    /**
     * Selects partner application, i.e. an executable instance of WarehouseManger application,
     * based upon closest proximity to customer zipcode and places order.
     * <p>
     * Note that this method utilizes the preferred signature which employs a non-blocking
     * invocation and which provides access to 'middleware' api, e.g. to {@link ActionContext}
     * and {@link MessageRT}.
     * 
     * 
     * @param myOperation
     * @param requestContext
     */
    public void placeOrder(MyOperation myOperation, ActionContext requestContext);

    /**
     * Selects partner application, i.e. an executable instance of WarehouseManger application,
     * based upon closest proximity to customer zipcode and places order.
     * <p>
     * Note that this method is used by the <code>OrderController</code> defined within
     * retail web application which is deployed as a part of this application (as a web
     * module), i.e. the method is not used by <code>SpringEngine</code>.
     * 
     * @param customerOrder
     *        the 'domain' object bound by spring framework and passed by controller
     * @return
     * @throws OutOfStockFault
     * @throws SystemError
     */
    public CustomerOrder placeOrder(CustomerOrder customerOrder) throws OutOfStockFault, SystemError;

    /**
     * Invoked by {@link Warehouse} which processes asynchronous replies sent in response to
     * schedule order requests. Completes outstanding placeOrder requests.
     * 
     * @param customerOrder
     * @param estimatedShipDate
     */
    public void placeOrderResponse(CustomerOrder customerOrder, Date estimatedShipDate);

    /**
     * An example method signature which does not rely upon classes defined within 'middleware'
     * api, i.e. classes defined within the EOA specification api.
     * <p>
     * Note method signature employs a blocking invocation. See
     * {@link #placeOrder(MyOperation, ActionContext)} for signature which provides access to
     * runtime request/response message defs, predefined message fragments and which
     * facilitates a non-blocking invocation, e.g. for potentially long running exchanges.
     * 
     * @param request
     * @return
     * @throws OrderNotFoundFault
     *         if order not found
     */
    public Element getOrderStatus(Element request) throws OrderNotFoundFault;
    
    /**
     * Selects partner application, i.e. an executable instance of WarehouseManger application,
     * based upon closest proximity to customer zipcode, places order and returns response.
     * <p>
     * Note that this method is used by the <code>OrderController</code> defined within
     * retail web application which is deployed as a part of this application (as a web
     * module), i.e. the method is not used by <code>SpringEngine</code>.
     * 
     * @param customerOrder the orderNumber
     */
    public Status getOrderStatus(String orderNumber) throws OrderNotFoundFault;

    /**
     * Receives shipment notification from warehouse that fulfilled the order. Completes order
     * by updating status to complete and by notifying customer via email that order has
     * shipped.
     * 
     * @param myOperation
     * @param requestContext
     */
    public void completeOrder(MyOperation myOperation, ActionContext requestContext);

}
